/* Copyright(C) 2014 Cobac.Net All rights reserved. */
/* M */

module syncgen(
    input               CLK,
    input               RST,
    output  reg         PCK,
    output  reg         VGA_HS,
    output  reg         VGA_VS,
    output  reg [9:0]   HCNT,
    output  reg [9:0]   VCNT
);

/* VGA(640~480)p[^ */
localparam HSC = 10'd800;
localparam HFP = 10'd16;
localparam HPW = 10'd96;
localparam HBP = 10'd48;

localparam VSC = 10'd525;
localparam VFP = 10'd10;
localparam VPW = 10'd2;
localparam VBP = 10'd33;

/* PixelNbN */
always @( posedge CLK ) begin
    PCK <= ~PCK;
end

/* JE^ */
wire hcntend = (HCNT==HSC-10'h001);

always @( posedge PCK ) begin
    if ( RST )
        HCNT <= 10'h000;
    else if ( hcntend )
        HCNT <= 10'h000;
    else
        HCNT <= HCNT + 10'h001;
end

/* JE^ */
always @( posedge PCK ) begin
    if ( RST )
        VCNT <= 10'h000;
    else if ( hcntend ) begin
        if ( VCNT==VSC-10'h001 )
            VCNT <= 10'h000;
        else
            VCNT <= VCNT + 10'h001;
    end
end

/* M */
wire [9:0] hsstart = HFP - 10'h001;
wire [9:0] hsend   = HFP + HPW - 10'h001;
wire [9:0] vsstart = VFP;
wire [9:0] vsend   = VFP + VPW;

always @( posedge PCK ) begin
    if ( RST )
        VGA_HS <= 1'b1;
    else if ( HCNT==hsstart )
        VGA_HS <= 1'b0;
    else if ( HCNT==hsend )
        VGA_HS <= 1'b1;
end

always @( posedge PCK ) begin
    if ( RST )
        VGA_VS <= 1'b1;
    else if ( HCNT==hsstart ) begin
        if ( VCNT==vsstart )
            VGA_VS <= 1'b0;
        else if ( VCNT==vsend )
            VGA_VS <= 1'b1;
    end
end

endmodule
